@@ -29,6 +29,7 @@
 #include <arpa/inet.h>
 
 #include "private.h"
+#include "useful.h"
 #include "dhcp.h"
 #include "dhcp-private.h"
 #include "utf8.h"
@@ -89,6 +90,9 @@ struct l_dhcp_lease *_dhcp_lease_parse_options(struct dhcp_message_iter *iter)
 				lease->broadcast = l_get_u32(v);
 			break;
 		case L_DHCP_OPTION_DOMAIN_NAME_SERVER:
+			if (lease->dns)
+				goto error;
+
 			if (l >= 4 && !(l % 4)) {
 				unsigned i = 0;
 
@@ -104,13 +108,22 @@ struct l_dhcp_lease *_dhcp_lease_parse_options(struct dhcp_message_iter *iter)
 			}
 			break;
 		case L_DHCP_OPTION_DOMAIN_NAME:
-			if (l < 1 || l > 253)
+			if (l < 1 || l > 253 || lease->domain_name)
 				goto error;
 
 			/* Disallow embedded NUL bytes. */
 			if (memchr(v, 0, l - 1))
 				goto error;
 
+			/*
+			 * RFC2132 doesn't say whether ending NULLs are present
+			 * or not.  However, section 2 recommends that trailing
+			 * NULLs should not be used but must not be treated
+			 * as an error
+			 */
+			if (l_get_u8(v + l - 1) == 0)
+				l -= 1;
+
 			if (!l_utf8_validate(v, l, NULL))
 				goto error;
 
@@ -168,12 +181,13 @@ error:
 static inline char *get_ip(uint32_t ip)
 {
 	struct in_addr addr;
+	char buf[INET_ADDRSTRLEN];
 
 	if (ip == 0)
 		return NULL;
 
 	addr.s_addr = ip;
-	return l_strdup(inet_ntoa(addr));
+	return l_strdup(inet_ntop(AF_INET, &addr, buf, INET_ADDRSTRLEN));
 }
 
 LIB_EXPORT char *l_dhcp_lease_get_address(const struct l_dhcp_lease *lease)
@@ -269,3 +283,11 @@ LIB_EXPORT uint32_t l_dhcp_lease_get_lifetime(const struct l_dhcp_lease *lease)
 
 	return lease->lifetime;
 }
+
+LIB_EXPORT const uint8_t *l_dhcp_lease_get_mac(const struct l_dhcp_lease *lease)
+{
+	if (unlikely(!lease))
+		return NULL;
+
+	return lease->mac;
+}
